Package: fail
Type: Package
Title: File Abstraction Interface Layer (FAIL)
Description: More comfortable interface to work with R data or source files
    in a key-value fashion.
Version: 1.3
Author: Michel Lang <michellang@gmail.com>
Maintainer: Michel Lang <michellang@gmail.com>
URL: https://github.com/mllg/fail
License: BSD_3_clause + file LICENSE
Imports: stats, utils, BBmisc, checkmate
Suggests: testthat
ByteCompile: yes
NeedsCompilation: no
Packaged: 2015-09-30 11:01:23 UTC; lang
Repository: CRAN
Date/Publication: 2015-10-01 00:21:25
Built: R 4.6.0; ; 2026-04-26 17:45:55 UTC; unix
